from reportlab.lib.pagesizes import legal
from reportlab.pdfgen import canvas
from reportlab.lib.units import inch
from reportlab.platypus import Image

def create_pdf(pdf_filename, images_filenames):
    reversed_legal = (14*inch,8.5*inch)

    c = canvas.Canvas(pdf_filename, pagesize=reversed_legal)
    for src in images_filenames:
        img = Image(src, width=14*inch, height=8.5*inch)
        img.hAlign = 'CENTER'
        img.wrapOn(c, *reversed_legal)
        img.drawOn(c,0,0)
        c.showPage()
    c.save()


from __future__ import division

def int_label_to_letter_label(number):
    result = []

    while True:
        number -= 1
        result.append(number%26)
        if number//26-1 < 0:
            break
        number //= 26

    return ''.join([chr(ord('A') + x) for x in reversed(result)])

def letter_label_to_int_label(letters):
    letters_ord = [ord(letter) - ord('A') + 1 for letter in list(letters)]
    result_list = [l * 26**i for i, l in enumerate(reversed(letters_ord))]

    result = 0
    for r in result_list:
    	result += r

    return result


from __future__ import division
import matplotlib
matplotlib.use('Agg')
import warnings
warnings.filterwarnings("ignore")
import matplotlib.pyplot as plt
import pylab
import math
from analysis.shared.util.labels import letter_label_to_int_label, int_label_to_letter_label
from analysis.annotate.report.composers.composer import Composer, PieChartsBuilder

class PiePageComposer(Composer):
    def __init__(self, annotation, title, group_name):
        self.group_name = group_name
        self.title = title
        self.annotation = annotation

    def run(self, left_lower_page, right_lower_page):
        matplotlib.rcParams['font.size'] = 14
        y_size = 1080*2
        x_size = (14 * y_size) / 8.5

        fig = pylab.figure(figsize=(x_size/100., y_size/100.), dpi=100.)

        # title
        axtitle = fig.add_axes([0.1, 0.9, 0.8, 0.01])
        axtitle.axis('off')
        pylab.title(self.title, fontsize=56, loc='left')
        axtitle.set_xticks([])
        axtitle.set_yticks([])

        self._make_piecharts_grid(fig)

        #time and page
        axid = fig.add_axes([0., 0., 1., 0.015])
        axid.axis('off')
        axid.text(0.005, 0.25, left_lower_page)
        axid.text(0.995, 0.25, right_lower_page, ha='right')
        axid.set_xticks([])
        axid.set_yticks([])

        # Display and save figure.
        filename = self.tmp_name()
        fig.savefig(filename, format='png')
        return filename

    def _make_piecharts_grid(self, fig):
        piecharts_builder = PieChartsBuilder()
        piecharts_builder.set_annotation(self.annotation)
        piecharts_builder.set_category_name(self.group_name)
        piecharts_builder.set_max_nb_tags(20)
        piecharts_builder.set_max_chars(30)
        piecharts_builder.set_show_nb_tags(True)

        piecharts = piecharts_builder.build()

        self._draw_piecharts(fig, piecharts)

        self._draw_legend(fig, piecharts)

    def _draw_piecharts(self, fig, piecharts):
        lenght = len(piecharts)

        # when's only two piecharts (All and A),
        # the piecharts are too high
        # inside the frame of the page
        if lenght == 2:
            start_at = 1
        else:
            start_at = 0

        dim = int(math.ceil(math.sqrt(lenght - start_at)))
        i_dim = dim
        j_dim = dim
        for cluster_id in range(start_at, lenght):
            i = (cluster_id - start_at) % i_dim
            j = (cluster_id - start_at) // i_dim
            axpie = fig.add_axes([0.1 + (i * (0.60/float(i_dim))),
                                  0.9 - ((j + 1) * (0.8/float(j_dim))),
                                  0.9 * (0.4845/float(i_dim)),
                                  0.9 * (0.798/float(j_dim))])
            patches, texts, autotexts = pylab.pie(piecharts.get_percents_for_cluster(cluster_id),
                                                  colors=piecharts.get_colors_for_cluster(cluster_id),
                                                  shadow=True,
                                                  autopct='%1.0f%%',
                                                  pctdistance=1.15)

            for autotext in autotexts:
                autotext.set_fontsize('small')

            pylab.title(piecharts.get_title_for_cluster(cluster_id), fontsize='large')

    def _draw_legend(self, fig, piecharts):
        axlegend = fig.add_axes([0.7, 0.1, 0.25, 0.8])
        axlegend.axis('off')
        legend = plt.legend(handles=piecharts.get_handles(), loc=9, title=self.group_name)
        plt.setp(legend.get_title(), fontsize='xx-large')
        axlegend.set_xticks([])
        axlegend.set_yticks([])



import pandas
import csv
from itertools import chain

class AnnotateTsv:
    def __init__(self, headers, matrix):
        self._matrix_index = {h:_Header.make_header(h, i, matrix)
                              for i, h in enumerate(headers)}
        self._matrix = matrix
        self.unique_identifier = headers[0]

    def list_headers(self):
        return self._matrix_index.keys()

    def make_filter(self, header_name, cond):
        header = self._matrix_index[header_name]
        return header.filter_index(cond)

    def list_items(self, header, *filters):
        row_indexes = set(range(len(self._matrix))).intersection(*filters)
        col_index = self._matrix_index[header].get_index_col()
        return [self._matrix[row_index, col_index] for row_index in row_indexes]

    @staticmethod
    def load_annotate_tsv(tsv_filename):
        with open(tsv_filename, 'r') as tsv_file:
            return AnnotateTsv.load_annotate_tsv_file(tsv_file)

    @staticmethod
    def load_annotate_tsv_file(tsv_file):
        dframe = pandas.read_csv(tsv_file,
                                 comment='#',
                                 delimiter='\t',
                                 index_col=False,
                                 keep_default_na=False)

        return AnnotateTsv(dframe.columns.values.tolist(), dframe.values)

    @staticmethod
    def annotation_to_tsv(annotation, categories_names, tsv_file):
        nb_clusters = annotation.get_nb_clusters()
        unique_identifier = annotation.get_metadata().unique_identifier
        filtered_cat_names = [x for x in categories_names if x != unique_identifier]

        description = '# {}'.format(annotation.description)
        datasets = annotation.annotation_datagrid_iter(filtered_cat_names)
        header = [unique_identifier, 'cluster({})'.format(nb_clusters)] + filtered_cat_names
        rows = chain([[description]], [header], datasets)
        
        AnnotateTsv._write_tsv(rows, tsv_file)

    @staticmethod
    def _write_tsv(rows, tsv_file):
        tsv_file.write(
            u'\n'.join(
                [u'\t'.join(
                    [u'{}'.format(cell)
                     for cell in row])
                 for row in rows]).encode('utf-8'))

class _Header:
    def __init__(self, header, index_col, values={}):
        self._header = header
        self._index_col = index_col
        self._values = values

    def get_index_col(self):
        return self._index_col

    def filter_index(self, cond):
        return [idx for value in self._values.values()
                if cond(value.get_name())
                for idx in value.get_indexes()]

    @staticmethod
    def make_header(header, index_col, matrix):
        values = {}
        for row_idx in range(len(matrix)):
            value_name = matrix[row_idx, index_col]
            
            if value_name in values:
                values[value_name].append(row_idx)
            else:
                values[value_name] = _Value(value_name, [row_idx])

        return _Header(header, index_col, values)

class _Value:
    def __init__(self, name, indexes=[]):
        self._name = name
        self._indexes = indexes

    def get_indexes(self):
        return self._indexes

    def get_name(self):
        return self._name

    def append(self, index):
        self._indexes.append(index)



from analysis.shared.clustering.hierarchical_clustering import HierarchicalClustering
from analysis.shared.clustering.clusters import Clusters, Cluster
from analysis.shared.categories import Categories, Category, Tags, Tag
from sklearn.metrics import adjusted_rand_score

class Breakdowns(list):
    def __init__(self, restriction_categories, *args):
        super(Breakdowns, self).__init__(*args)
        self.restriction_categories = restriction_categories

    def get_restriction_categories(self):
        return self.restriction_categories

    @staticmethod
    def make_breakdowns(matrix, metadata, breakdown_categories, restriction_categories, linkage_method):
        categories = Categories.make_categories(metadata,
                                                matrix.get_file_names(),
                                                set(breakdown_categories + restriction_categories))

        return Breakdowns(restriction_categories,
                          [Breakdown.make_breakdown_of_all(matrix,
                                                           categories,
                                                           restriction_categories,
                                                           linkage_method)] +
                          [Breakdown.make_breakdown(matrix,
                                                    metadata,
                                                    br_cat,
                                                    restriction_categories,
                                                    br_tag,
                                                    linkage_method)
                           for br_cat in breakdown_categories
                           for br_tag in categories.obtain_sorted_tags(br_cat)])

class Breakdown:
    def __init__(self, breakdown_name, dataset_size, sub_breakdowns):
        self.breakdown_name = breakdown_name
        self.dataset_size = dataset_size
        self.sub_breakdowns = sub_breakdowns

    def get_name(self):
        return self.breakdown_name

    def get_dataset_size(self):
        return self.dataset_size

    def get_aris(self):
        return self.sub_breakdowns.get_aris()

    def get_nbs_clusters(self):
        return self.sub_breakdowns.get_nbs_clusters()

    @staticmethod
    def make_breakdown(matrix, metadata, breakdown_category, restriction_categories, tag, linkage_method):
        breakdown_name = 'breakdown:{}:{}'.format(breakdown_category, tag.get_name())
        sub_matrix = matrix.sub_matrix(tag.get_file_names())
        sub_categories = Categories.make_categories(metadata,
                                                    tag.get_file_names(),
                                                    restriction_categories)
        cluster_maker = HierarchicalClustering(sub_matrix, linkage_method)

        return Breakdown(breakdown_name,
                         len(sub_matrix),
                         SubBreakdowns.make_sub_breakdowns(sub_matrix,
                                                           sub_categories,
                                                           restriction_categories,
                                                           cluster_maker))

    @staticmethod
    def make_breakdown_of_all(matrix, categories, restriction_categories, linkage_method):
        cluster_maker = HierarchicalClustering(matrix, linkage_method)
        return Breakdown('all',
                         len(matrix),
                         SubBreakdowns.make_sub_breakdowns(matrix,
                                                           categories,
                                                           restriction_categories,
                                                           cluster_maker))

class SubBreakdowns(list):
    def __init__(self, *args):
        super(SubBreakdowns, self).__init__(*args)

    def get_aris(self):
        return [sb.get_ari() for sb in self]

    def get_nbs_clusters(self):
        return [sb.get_nb_clusters() for sb in self]

    @staticmethod
    def make_sub_breakdowns(matrix, categories, restriction_categories, cluster_maker):
        return SubBreakdowns([SubBreakdown.make_sub_breakdown(matrix,
                                                              categories[category_name],
                                                              cluster_maker)
                              for category_name in restriction_categories])

class SubBreakdown:
    def __init__(self, ari_value, nb_clusters):
        self.ari_value = ari_value
        self.nb_clusters = nb_clusters

    def get_ari(self):
        return self.ari_value

    def get_nb_clusters(self):
        return self.nb_clusters

    @staticmethod
    def make_sub_breakdown(matrix, category, cluster_maker):
        flat_clusters = cluster_maker \
                        .run(len(category)) \
                        .make_flat_clusters(matrix)
        cat_flat_clusters = category.make_flat_clusters(matrix)

        # The data sources needs to be align
        # In our case we use the matrix order
        return SubBreakdown(adjusted_rand_score(flat_clusters, cat_flat_clusters),
                            len(category))



from __future__ import absolute_import, division, print_function

from analysis.shared.metadata import Metadata, MetadataFactory
import re

def load_metadata(filenames, unique_identifier):
    metadatas = [Metadata.parse_metadatafile(input_json, unique_identifier) for input_json in filenames]
    return sum(metadatas, MetadataFactory.empty(unique_identifier))

def load_patterns(pattern, is_file):
    if is_file:
        with open(pattern, 'r') as text_file:
            return [line.strip() for line in text_file]
    else:
        return [pattern]

def grep(metadata, pattern, is_regex, is_file, is_inverted):
    patterns = load_patterns(pattern, is_file)

    if is_regex:
        return metadata.match(patterns, is_inverted)
    else:
        return metadata.find(patterns, is_inverted)

def run(inputs, pattern, is_regex, is_file, is_inverted, unique_identifier):
    metadata = load_metadata(inputs, unique_identifier)

    print(str(grep(metadata, pattern, is_regex, is_file, is_inverted)))



class Categories(dict):
    def __init__(self, categories_dict):
        super(Categories, self).__init__(categories_dict)

    def obtain_file_names_with_tag_name(self, tag_name):
        return set(file_name for ca in self.values() 
                   for file_name in ca.obtain_file_names_with_tag_name(tag_name))

    def obtain_size_of_tags_in_category(self, category_name):
        return self[category_name].obtain_size_of_tags()

    def obtain_size_of_tags_from_tag_names(self, category_name, tag_names, file_names):
        return self[category_name].obtain_size_of_tags_from_tag_names(tag_names, file_names)

    def obtain_file_names_tag_names(self, category_name):
        return self[category_name].obtain_file_names_tag_names()

    def obtain_sorted_tags(self, category_name):
        return self[category_name].obtain_sorted_tags()

    @staticmethod
    def make_categories(metadata, file_names, categories_names):
        return Categories({category_name: Category.make_category(metadata, file_names, category_name)
                           for category_name in categories_names})

class Category:
    def __init__(self, name, tags):
        self.name = name
        self.tags = tags

    def get_name(self):
        return self.name

    def get_tags(self):
        return self.tags

    def obtain_file_names_with_tag_name(self, tag_name):
        return self.tags.get(tag_name, Tag(tag_name, [])).get_file_names()

    def obtain_size_of_tags(self):
        return self.tags.obtain_size_of_tags()

    def obtain_size_of_tags_from_tag_names(self, tag_names, file_names):
        return self.tags.obtain_size_of_tags_from_tag_names(tag_names, set(file_names))

    def obtain_file_names_tag_names(self):
        return self.tags.obtain_file_names_tag_names()

    def obtain_sorted_tags(self):
        return sorted(sorted(self.tags.values(), key=lambda t: t.get_name()), key=lambda t: len(t), reverse=True)

    def make_flat_clusters(self, matrix):
        fn_flc = {}
        for idx, tag in enumerate(self.tags.values()):
            for file_name in tag.get_file_names():
                fn_flc[file_name] = idx

        return [fn_flc[fn] for fn in matrix.get_file_names()]

    def __len__(self):
        return len(self.tags)

    @staticmethod
    def make_category(metadata, file_names, category_name):
        return Category(category_name, Tags.make_tags(metadata.extract_tag_names_file_names(file_names, category_name)))

class Tags(dict):
    def __init__(self, tags_dict):
        super(Tags, self).__init__(tags_dict)

    def obtain_size_of_tags(self):
        return [(k, len(tag)) for k, tag in self.items()]

    def obtain_size_of_tags_from_tag_names(self, tag_names, file_names):
        return sum([self[tag_name].obtain_size_of_intersection(file_names)
                    for tag_name in tag_names
                    if tag_name in self])

    def obtain_file_names_tag_names(self):
        return dict([file_names_tag_names for tag in self.values() for file_names_tag_names in tag.obtain_file_names_tag_names()])

    @staticmethod
    def make_tags(tags_file_names):
        return Tags({tag: Tag(tag, file_names) for tag, file_names in tags_file_names.items()})

class Tag:
    def __init__(self, name, file_names):
        self.name = name
        self.file_names = file_names

    def get_name(self):
        return self.name

    def get_file_names(self):
        return self.file_names

    def obtain_file_names_tag_names(self):
        return [(file_name, self.name) for file_name in self.file_names]

    def obtain_size_of_intersection(self, file_names):
        return len(file_names.intersection(self.file_names))

    def __len__(self):
        return len(self.file_names)


import logging

logging.basicConfig()
logger = logging.getLogger(__name__)

DESCRIPTION_CLUSTERING_ALGORITHM = 'clusteringAlgorithm={}'
DESCRIPTION_LINKAGE_METHOD = 'linkageMethod={}'

class Annotation:
    def __init__(self, matrix, metadata, comparison):
        self.matrix = matrix
        self.metadata = metadata
        self.comparison = comparison

        if not metadata.key_intersection(matrix.get_file_names()):
            logger.warning("There's no match between the matrix and the metadata.")

    @property
    def description(self):
        description_list = []

        matrix_description = self.matrix.description

        if matrix_description:
            description_list.append(matrix_description)

        description_list.extend(
            [
                DESCRIPTION_CLUSTERING_ALGORITHM.format(self.clustering_algorithm),
                DESCRIPTION_LINKAGE_METHOD.format(self.linkage_method)
            ]
        )

        return ', '.join(description_list)

    def obtain_size_of_tags_in_category(self, category_name):
        return self.comparison.obtain_size_of_tags_in_category(category_name)

    def obtain_file_names_with_tag_name(self, tag_name):
        return self.comparison.obtain_file_names_with_tag_name(tag_name)

    def obtain_cluster_name(self, cluster_id):
        return self.comparison.obtain_cluster_name(cluster_id)

    def obtain_size_of_tags_from_tag_names(self, cluster_id, category_name, tag_names):
        return self.comparison.obtain_size_of_tags_from_tag_names(cluster_id, category_name, tag_names)

    def obtain_file_names_tag_names(self, category_name):
        return self.comparison.obtain_file_names_tag_names(category_name)

    def obtain_ordered_tag_names(self, category_name):
        file_names_tag_names = self.comparison.obtain_file_names_tag_names(category_name)
        return [file_names_tag_names[file_name]
                for file_name in self.matrix.get_file_names()]

    def make_flat_clusters(self):
        return self.comparison.make_flat_clusters(self.matrix)

    def obtain_clusters(self):
        return self.comparison.clusters

    @property
    def linkage_method(self):
        return self.comparison.clusters.linkage_method

    @property
    def clustering_algorithm(self):
        return self.comparison.clusters.clustering_algorithm

    def get_nb_clusters(self):
        return self.comparison.clusters.count

    def get_matrix(self):
        return self.matrix

    def get_file_names(self):
        return self.matrix.get_file_names()

    def get_metadata(self):
        return self.metadata

    def __len__(self):
        return len(self.matrix)

    def annotation_datagrid_iter(self, categories_names):
        for file_name in self.matrix.get_file_names():
            yield ([file_name, self.comparison.obtain_cluster_name_by_file_name(file_name)] + 
                   self.metadata.obtain_formated_dataset(file_name, categories_names))

class Comparison:
    ALL = 0
    def __init__(self, clusters, categories):
        self._clusters = clusters
        self._categories = categories

    @property
    def clusters(self):
        return self._clusters

    @property
    def categories(self):
        return self._categories

    def obtain_file_names_with_tag_name(self, tag_name):
        return self.categories.obtain_file_names_with_tag_name(tag_name)

    def obtain_size_of_tags_in_category(self, category_name):
        return self.categories.obtain_size_of_tags_in_category(category_name)

    def obtain_cluster_name(self, cluster_id):
        if cluster_id == self.ALL:
            return 'All'
        else:
            return self.clusters[cluster_id].get_name()

    def obtain_size_of_tags_from_tag_names(self, cluster_id, category_name, tag_names):
        if cluster_id == self.ALL:
            file_names = self.clusters.get_file_names()
        else:
            file_names = self.clusters[cluster_id].get_file_names()
        return self.categories.obtain_size_of_tags_from_tag_names(category_name, tag_names, file_names)

    def obtain_file_names_tag_names(self, category_name):
        return self.categories.obtain_file_names_tag_names(category_name)

    def obtain_cluster_name_by_file_name(self, file_name):
        return self.clusters.obtain_cluster_name(file_name)

    def make_flat_clusters(self, matrix):
        return self._clusters.make_flat_clusters(matrix)



import re

class StringBuilder():
	def __init__(self, template):
		self.template = template
		self.params = {}

	def build(self):
		tags = re.findall('%\\((.*?)\\)', self.template)
		template_tags = [(tag, self.params.get(tag, 'None')) for tag in tags]

		transformed = self.template
		for tag, value in template_tags:
			transformed = re.sub('%\\((' + tag +')\\)', str(value), transformed)

		return transformed

	def set(self, key, value):
		self.params[key] = value


from analysis.shared.categories import Categories
from analysis.shared.clustering.clusters import Clusters
from analysis.shared.clustering.hierarchical_clustering import HierarchicalClustering
from analysis.shared.clustering.scipypatch import ScipyPatch
from analysis.shared.matrix import Matrix
from analysis.shared.metadata import Metadata
from analysis.config import VERSION

from .annotation import Annotation, Comparison
from .report import pdf
from .report.annotate_tsv import AnnotateTsv
from .report.rendering import Rendering

class AnnotateApp:
    def __init__(self, params):
        self.matrix_filename = params['matrix_filename']
        self.metadata_filename = params['metadata_filename']
        self.groups = params['groups']
        self.title = params['title']
        self.linkage_method = params['linkage_method']
        self.desired_nb_clusters = params['desired_nb_clusters']
        self.pdf_file = params['pdf_file']
        self.tsv_file = params['tsv_file']
        self.reordered_matrix_filename = params['reordered_matrix_filename']
        self.seed = params['seed']
        self.mds = params['mds']
        self.rescale = params['rescale']
        self.md5sum = params['md5sum']

    def run(self):
        #load files
        matrix = self._load_matrix()
        data = self._load_metadata()

        #base data
        restrictions = ['md5sum', 'file_path', 'file_name', 'id', 'virtual', 'assembly']
        ordered_categories = ['assay', 'publishing_group', 'cell_type']
        groups = data.make_usable_categories(matrix.get_file_names(),
                                             self.groups,
                                             restrictions,
                                             ordered_categories)

        # clustering
        clusters = HierarchicalClustering.make_clusters(matrix, self.linkage_method, self.desired_nb_clusters)

        # Categories
        categories = Categories.make_categories(data, matrix.get_file_names(), groups)

        # Comparison
        remapped_clusters = ScipyPatch.rename_clusters(clusters, matrix)
        comparison = Comparison(remapped_clusters, categories)

        matrix = ScipyPatch.reorder_matrix(remapped_clusters, matrix)

        # Annotation
        annotation = Annotation(matrix, data, comparison)

        # pdf export
        if self.pdf_file is not None:
            self._export_pdf(groups, annotation)

        # tsv export
        if self.tsv_file is not None:
            self._export_tsv(annotation)

        if self.reordered_matrix_filename is not None:
            self._write_reordered_matrix(annotation)

    def _export_pdf(self, groups, annotation):
        cr = Rendering("epiGeEC-Annotate version {}".format(VERSION))

        self.title.set('nb_clusters', annotation.get_nb_clusters())
        files = cr.render(annotation, self.title.build(), self.seed, self.mds, groups, self.rescale)

        pdf.create_pdf(self.pdf_file, files)

    def _export_tsv(self, annotation):
        metadata = annotation.get_metadata()

        restrictions = ['file_path', 'id', 'virtual']
        ordered_categories = ['assay', 'cell_type', 'publishing_group', 'assembly']
        categories_names = metadata.make_usable_categories(annotation.get_file_names(),
                                                          None,
                                                          restrictions,
                                                          ordered_categories)

        with open(self.tsv_file, 'w') as tsv_file:
            AnnotateTsv.annotation_to_tsv(annotation, categories_names, tsv_file)

    def _write_reordered_matrix(self, annotation):
        matrix = annotation.get_matrix()
        with open(self.reordered_matrix_filename, 'w') as rmat_f: 
            matrix.write(rmat_f)

    def _load_matrix(self):
        return Matrix.parse_matrixfile(self.matrix_filename)

    def _load_metadata(self):
        if self.md5sum:
            unique_identifier = 'md5sum'
        else:
            unique_identifier = 'file_name'

        return Metadata.parse_metadatafile(self.metadata_filename, unique_identifier)



from analysis.ari.ariapp import AriApp

class AriAppBuilder:
    def __init__(self):
        self.params = {
            'matrix_filename':None,
            'metadata_filename':None,
            'linkage_method':None,
            'breakdown_categories':None,
            'restriction_categories':None,
            'title':None,
            'ignore_user':None,
            'md5sum':False
        }

    def set_matrix_filename(self, matrix_filename):
        self.params['matrix_filename'] = matrix_filename

    def set_metadata_filename(self, metadata_filename):
        self.params['metadata_filename'] = metadata_filename

    def set_linkage_method(self, linkage_method):
        self.params['linkage_method'] = linkage_method

    def set_breakdown_categories(self, breakdown_categories):
        self.params['breakdown_categories'] = breakdown_categories

    def set_restriction_categories(self, restriction_categories):
        self.params['restriction_categories'] = restriction_categories

    def set_title(self, title):
        self.params['title'] = title

    def set_ignore_user(self, ignore_user):
        self.params['ignore_user'] = ignore_user

    def set_md5sum(self, md5sum):
        self.params['md5sum'] = md5sum

    def build(self):
        for k, p in self.params.items():
            if p is None:
                raise Exception('The parameter:\'' + k + '\' is not specified.')

        return AriApp(self.params)



from __future__ import absolute_import, division, print_function

import json
import sys
import argparse

class IhecJson(object):
    """
    """
    def __init__(self):
        self.content = []
        self.count = 0

    @staticmethod
    def epigeec_format(ihec_json):
        """
        """
        ihec = IhecJson()
        ihec._load(ihec_json)
        return ihec.epigeec_dict()

    def load(self, ihec_json_file):
        """
        """
        self._load(json.load(ihec_json_file))

    def _load(self, ihec_json):
        """
        """
        hub_description = ihec_json['hub_description']
        datasets = ihec_json['datasets']

        publishing_group = hub_description["publishing_group"]
        releasing_group = hub_description.get("releasing_group", publishing_group)
        assembly = hub_description["assembly"]
        for dataset_name, data in datasets.items():
            ihecdata = data.get("ihec_data_portal", {})
            assay = ihecdata.get("assay", "N/A")
            assay_category = ihecdata.get("assay_category", "N/A")
            cell_type = ihecdata.get("cell_type", "N/A")
            cell_type_category = ihecdata.get("cell_type_category", "N/A")
            publishing_group = ihecdata.get('publishing_group', 'N/A')
            releasing_group = ihecdata.get('releasing_group', 'N/A')

            #signal type priority
            for signal_type in ["methylation_profile", "signal_forward", "signal_unstranded", "signal"]:
                signal_data = data.get("browser", {}).get(signal_type, [{}])[0]
                if signal_data:
                    break

            md5sum = signal_data.get("md5sum")
            if md5sum is None:
                continue
            unique_id = str(self.count)
            self.count += 1
            parsed_dataset = {
                "assembly": assembly,
                "publishing_group": publishing_group,
                "releasing_group": releasing_group,
                "assay": assay,
                "assay_category": assay_category,
                "cell_type": cell_type,
                "cell_type_category": cell_type_category,
                "file_name": dataset_name,
                "md5sum": md5sum,
                "id": unique_id,
            }
            parsed_dataset["virtual"] = (signal_type == "signal_forward")
            self.content.append(parsed_dataset)

    def epigeec_dict(self):
        """
        """
        return {"datasets":self.content}

    def __str__(self):
        """
        """
        return json.dumps(self.epigeec_dict())


def main():
    """
    """
    ihec_json_paths = sys.argv[1:]
    ihec_json = IhecJson()
    for ihec_json_path in ihec_json_paths:
        ihec_json.load(open(ihec_json_path))
    print(ihec_json)

if __name__ == "__main__":
    main()


from __future__ import absolute_import, division, print_function

import argparse
from .launcher import AnnotateLauncher

class RangeType(object):
    def __init__(self, minv, maxv):
        self.minv = minv
        self.maxv = maxv

    def __call__(self, x):
        value = self.int_type(x)

        if value < self.minv or value > self.maxv:
            raise argparse.ArgumentTypeError('The value "{}" is not between {:,} and {:,}.'.format(value, self.minv, self.maxv))

        return value

    def int_type(self, x):
        try:
            return int(x)
        except ValueError:
            raise argparse.ArgumentTypeError('{!r} is not an integer.'.format(x))

class AnnotateParser(object):
    @staticmethod
    def add_parser(subparsers):
        annotate_parser = subparsers.add_parser(
            'annotate',
            description='Annotate clusters of the matrix',
            help='Annotate clusters of the matrix')
        annotate_parser.set_defaults(func=AnnotateLauncher.run)

        annotate_parser.add_argument('matrix', help='The input matrix tsv file.', type=str)
        annotate_parser.add_argument('--rmat', help='Write a reordered matrix.', type=str)
        annotate_parser.add_argument('--pdf', help='The output pdf file.', type=str)
        annotate_parser.add_argument('--tsv', help='The output tsv file.', type=str)
        annotate_parser.add_argument(
            '--title',
            help='The title of the report.',
            default='epiGeEC Annotate Matrix, k=%(nb_clusters) clusters',
            type=str)
        annotate_parser.add_argument(
            '--mds',
            help='Add a Multidimensional scaling figure (MDS) inside the pdf file.',
            action='store_true')
        annotate_parser.add_argument(
            '--seed',
            help='The seed number to generate the MDS [0-4294967295].',
            type=RangeType(0, 4294967295))
        annotate_parser.add_argument('metadata', help='The input metadata json file.', type=str)
        annotate_parser.add_argument(
            '-k',
            help='The numbers of clusters [1-4294967295]. If not specified, an optimal clustering is used.',
            type=RangeType(1, 4294967295))
        annotate_parser.add_argument(
            '--linkage',
            help='The linkage method. By default: "average" Options: "average", "complete", "weighted" or "single".',
            default='average',
            choices=['average', 'complete', 'weighted', 'single'],
            type=str)
        annotate_parser.add_argument(
            'categories',
            nargs='*',
            help='The categories used inside the reports. If not specified, it will use all categories in metadata.',
            default=[],
            metavar='category',
            type=str)
        annotate_parser.add_argument(
            '--rescale',
            help='Rescale the color of the heatmap with the min and max of the matrix.',
            action='store_true')
        annotate_parser.add_argument(
            '--md5sum',
            help="Set 'md5sum' as the unique identifier for files instead of file_names.",
            action='store_true')



from __future__ import absolute_import, division, print_function

import argparse

from .config import VERSION, NAME, DESCRIPTION
from .annotate.parser import AnnotateParser
from .ari.parser import EvaluateParser
from .slice.parser import SliceFilenameParser, SliceClusterParser
from .util.parser import CatParser, GrepParser

class InputParser(object):
    @staticmethod
    def parse_args(args):
        parser = argparse.ArgumentParser(prog=NAME, description = DESCRIPTION)
        parser.add_argument('-v', '--version', action='version', version='%(prog)s {0}'.format(VERSION))
        subparsers = parser.add_subparsers(help = "Sub-command help.")

        AnnotateParser.add_parser(subparsers)
        EvaluateParser.add_parser(subparsers)
        SliceFilenameParser.add_parser(subparsers)
        SliceClusterParser.add_parser(subparsers)
        CatParser.add_parser(subparsers)
        GrepParser.add_parser(subparsers)
        
        return parser.parse_args(args)



import matplotlib
matplotlib.use('Agg')
import warnings
warnings.filterwarnings("ignore")
import numpy
import matplotlib.pyplot as plt
import scipy.cluster.hierarchy as sch
import pylab
from textwrap import TextWrapper
from analysis.annotate.report.composers.composer import Composer, USER, LegendMaker, PieChartsBuilder
from analysis.shared.clustering.scipypatch import ScipyPatch
from analysis.shared.util.labels import letter_label_to_int_label, int_label_to_letter_label

class FirstPageComposer(Composer):
    def __init__(self, annotation, title, ordered_group_names, rescale_heatmap):
        self.annotation = annotation
        self.title = title
        self.ordered_group_names = ordered_group_names
        self.rescale_heatmap = rescale_heatmap

    def run(self, left_lower_page, right_lower_page):
        ordered_matrix = self.annotation.get_matrix()

        if len(self.annotation) < 1785:
            scaling = 1785/float(len(self.annotation))
        else:
            scaling = 1

        matplotlib.rcParams['font.size'] = 14 * scaling * len(self.annotation)/1785
        m = len(self.annotation) * scaling
        n = (14 * m) / 8.5

        x_size = int(n * 5/4.)
        y_size = int(m * 5/4.)

        fig = pylab.figure(figsize=(x_size/100., y_size/100.), dpi=100.)
        axdendro = fig.add_axes([0.0,0.1,0.1,0.8])
        axdendro.axis('off')

        ScipyPatch.make_dendrogram(self.annotation.obtain_clusters(), self.annotation.get_matrix())
        axdendro.set_xticks([])
        axdendro.set_yticks([])

        # Plot distance matrix.
        axmatrix = fig.add_axes([0.11,0.1,0.4857142857,0.8])
        axmatrix.axis('off')

        if self.rescale_heatmap:
            vmin, vmax = None, None
        else:
            vmin, vmax = -1, 1

        im = axmatrix.matshow(ordered_matrix.get_matrix(), interpolation='nearest', cmap='RdBu_r', origin='upper', vmin=vmin, vmax=vmax)
        axmatrix.set_xticks([])
        axmatrix.set_yticks([])

        # User matrix line
        users_headers = self.annotation.obtain_file_names_with_tag_name(USER)
        users_line_pos = [-i for i, h in enumerate(ordered_matrix.get_file_names()) if h in users_headers]
        axuserline = fig.add_axes([0.5965, 0.1, 0.005, 0.8])
        axuserline.axis('off')
        # plt.eventplot(users_line_pos, orientation='vertical', colors=[self.user_color])
        plt.plot([1 for _ in range(len(users_line_pos))], users_line_pos, 'ro', markersize=matplotlib.rcParams['font.size']/2., color=LegendMaker.USER_COLOR)
        pylab.xlim([0.5, 1.5])
        pylab.ylim([-len(ordered_matrix)+1, 0])
        axuserline.set_xticks([])
        axuserline.set_yticks([])

        # Plot colorbar.
        axcolor = fig.add_axes([0.11,0.905,0.4857142857,0.025])
        pylab.colorbar(im, cax=axcolor, orientation='horizontal')
        axcolor.xaxis.set_ticks_position('top')
        pylab.title(self.title, fontsize=(matplotlib.rcParams['font.size'] * 4.2), y=2, loc='left')

        # Pie charts
        self._draw_aligned_piecharts(fig)

        # cluster id
        axcluster = fig.add_axes([0.102,0.1,0.005,0.8])
        axcluster.axis('off')
        self._align_cluster_labels(axcluster, scaling)
        axcluster.set_xticks([])
        axcluster.set_yticks([])

        # description
        self._draw_description(fig)

        #time and page
        axid = fig.add_axes([0., 0., 1., 0.015])
        axid.axis('off')
        axid.text(0.005, 0.25, left_lower_page)
        axid.text(0.995, 0.25, right_lower_page, ha='right')
        axid.set_xticks([])
        axid.set_yticks([])

        # Display and save figure.
        filename = self.tmp_name()
        fig.savefig(filename, format='png')
        return filename

    def _draw_description(self, fig):
        formatted_description = '\n'.join(
            TextWrapper(
                width=140
            ).wrap(
                self.annotation.description))
        axdescription = fig.add_axes([0.11,0.015,0.4857142857,0.075])
        axdescription.text(0, 1, formatted_description, va='top', ha='left', clip_on=True, fontsize='large')
        axdescription.axis('off')
        axdescription.set_xticks([])
        axdescription.set_yticks([])

    def _compute_color_threshold(self, linkage_matrix, threshold):
        return linkage_matrix[-(threshold-1),2]

    def _draw_aligned_piecharts(self, fig):
        for column, group in enumerate(self.ordered_group_names[:3]):
            piecharts_builder = PieChartsBuilder()
            piecharts_builder.set_annotation(self.annotation)
            piecharts_builder.set_category_name(group)
            piecharts_builder.set_max_nb_tags(12)
            piecharts_builder.set_max_chars(30)

            piecharts = piecharts_builder.build()
        
            self._draw_piecharts_lines(fig, piecharts, column)
            self._draw_legend(fig, piecharts, column)

    def _draw_legend(self, fig, piecharts, col_pos):
        if len(piecharts) <= 20:
            axlegend = fig.add_axes([0.60 + (col_pos * 0.1), 0.0  , 0.1, 0.21])
            axlegend.axis('off')
            axlegend.set_xticks([])
            axlegend.set_yticks([])
            legend = plt.legend(handles=piecharts.get_handles(),
                                loc=2,
                                borderaxespad=0.,
                                title=piecharts.get_category_name())
            plt.setp(legend.get_title(), fontsize='xx-large')

    def _draw_piecharts_lines(self, fig, piecharts, column):
        nb_piecharts = len(piecharts)

        if nb_piecharts <= 10:
            line_a = reversed(range(nb_piecharts))
            line_b = range(0)
            stretch = 10 / float(nb_piecharts)
        elif nb_piecharts <= 20:
            side = nb_piecharts % 2 # 0 or 1
            line_a = reversed(range(1 - side, nb_piecharts, 2))
            line_b = reversed(range(0 + side, nb_piecharts, 2))
            stretch = 20 / float(nb_piecharts)

        if nb_piecharts <= 20:
            distance_between_charts = 0.07 * stretch
            first_axes = [0.60 + (column * 0.1), 0.215, 0.034, 0.056]
            second_axes = [0.64 + (column * 0.1), 0.235 + 0.020 * stretch, 0.034, 0.056]

            self._draw_piecharts_line(fig, piecharts, line_a, first_axes, distance_between_charts)
            self._draw_piecharts_line(fig, piecharts, line_b, second_axes, distance_between_charts)

    def _draw_piecharts_line(self, fig, piecharts, cluster_ids, axes, pos_inc):
        for pos, cluster_id in enumerate(cluster_ids):
            fig.add_axes([axes[0], axes[1] + (pos * pos_inc), axes[2], axes[3]])
            pylab.pie(piecharts.get_percents_for_cluster(cluster_id),
                      colors=piecharts.get_colors_for_cluster(cluster_id),
                      shadow=True)
            pylab.title(piecharts.get_title_for_cluster(cluster_id), y=0.95)

    def _align_cluster_labels(self, ax, scaling):
        total = len(self.annotation)
        clusters = self.annotation.obtain_clusters()

        total_px = total * scaling

        # caculate pixels position for text and repositionning
        px_pos = self._compute_text_pos(clusters, total_px, scaling)

        # compile text
        for cluster_id in range(1, len(clusters) + 1):
            ax.text(0.0, px_pos[cluster_id] / float(total_px), clusters[cluster_id].get_name(), fontsize='medium', family='monospace')

    def _compute_text_pos(self, clusters, total_px, scaling):
        before = total_px
        pos_before = total_px
        px_pos = {}
        for cluster_id in range(1, len(clusters) + 1):
            px_pos[cluster_id] = before - (len(clusters[cluster_id]) * scaling / 2)
            diff = (pos_before) - (px_pos[cluster_id] + matplotlib.rcParams['font.size'] + 2)
            if (diff < 0):
                px_pos[cluster_id] += diff
            pos_before = px_pos[cluster_id]
            before -= len(clusters[cluster_id]) * scaling

        return px_pos



from __future__ import absolute_import, division, print_function

import sys, argparse
from analysis.shared.metadata import Metadata, MetadataFactory

def run(inputs, unique_identifier):
    metadatas = [Metadata.parse_metadatafile(input_json, unique_identifier) for input_json in inputs]
    print(str(sum(metadatas, MetadataFactory.empty(unique_identifier))))



import numpy, pandas
import logging
from analysis.shared.exception.geecanalysiserror import GeecAnalysisError

logging.basicConfig()
logger = logging.getLogger(__name__)

class Not2dMatrixError(GeecAnalysisError):
    pass

class NotSquareMatrixError(GeecAnalysisError):
    pass

class NotSymetricMatrixError(GeecAnalysisError):
    pass

class MatrixValueError(GeecAnalysisError):
    pass

class MatrixIOError(GeecAnalysisError):
    pass

class Matrix:

    def get_matrix(self):
        return self.matrix

    def get_file_names(self):
        return self.file_names

    @property
    def description(self):
        return self._description

    def __init__(self, matrix, file_names, description):
        self.file_names = file_names
        self.headers_dict = {}
        self.matrix = matrix
        self._description = description

        if self.matrix.dtype != numpy.float64:
            raise MatrixValueError("The matrix should be float type.")

        if len(self.matrix.shape) != 2:
            raise Not2dMatrixError("The matrix is not 2 dimentions.")

        if not self.is_square_matrix():
            raise NotSquareMatrixError("The matrix is not a square matrix.")

        if not self.is_symetric():
            raise NotSymetricMatrixError("The matrix is not symetric.")

        for i, file_name in enumerate(self.file_names):
            self.headers_dict[file_name] = i

    def write(self, stream):
        stream.write(str(self))

    @staticmethod
    def parse_matrixfile(matrix_filename):
        with open(matrix_filename, 'r') as matrix_file:
            return Matrix.parse_matrix(matrix_file)

    @staticmethod
    def parse_matrix(matrix_file):
        dframe = pandas.read_csv(matrix_file, delimiter='\t', index_col=0)

        matrix = numpy.nan_to_num(dframe.values)
        file_names = dframe.columns.values.tolist()
        description = dframe.index.name

        if description is None:
            description = ''

        if Matrix.is_float_file_names(file_names):
            logger.warning("The filenames heading the matrix seams to be floats")

        try:
            return Matrix(matrix, file_names, description)
        # Fix the problem of pandas in python 2.7.5
        # with empty lines at the end of file
        # generate lines of zeros
        except NotSquareMatrixError:
            return Matrix(Matrix.remove_zeros_line(matrix), file_names, description)

    @staticmethod
    def remove_zeros_line(matrix):
        return matrix[numpy.where(matrix.any(axis=1))[0]]

    @staticmethod
    def is_float(num):
        try:
            float(num)
            return True
        except ValueError:
            return False

    @staticmethod
    def is_float_file_names(file_names):
        """
        Test if all filenames are float.
        """
        return all([Matrix.is_float(num) for num in file_names])

    def is_symetric(self):
        """
        return true if the matrix is symetric
        """
        return numpy.allclose(self.matrix, self.matrix.T, atol=1e-8)

    def is_square_matrix(self):
        """
        Test if the 2 axes are the same dimention.
        """
        return self.matrix.shape[0] == self.matrix.shape[1]

    def sub_matrix(self, file_names):
        """
        args: The headers of rows and colums to obtain in the sub matrix
        return: The sub matrix with only the desired items from the headers
        """
        
        indexes = [self.headers_dict[x] for x in file_names if x in self.headers_dict]
        headers = [self.file_names[i] for i in indexes]
        matrix = self.matrix[numpy.ix_(indexes, indexes)]

        return Matrix(matrix, headers, self.description)

    def __iter__(self):
        return numpy.nditer(self.matrix)

    def to_distance(self):
        return Matrix(1 - self.matrix, list(self.file_names), self.description)

    def __str__(self):
        """
        return formatted GeecMatrix
        """
        return self.__repr__()

    def __repr__(self):
        """
        return formatted GeecMatrix
        """
        description = self.description
        if description is None:
            description = ''

        #header line
        header_str = description + '\t' + '\t'.join(self.file_names)
        matrix_str = ''

        for (i, lines) in enumerate(self.matrix):
            matrix_str += '\n' + self.file_names[i] + '\t' + '\t'.join(str(e) for e in lines)

        return header_str + matrix_str

    def __len__(self):
        return len(self.matrix)

    def __add__(self, other):
        return self.matrix + other.matrix

    def __sub__(self, other):
        return self.matrix - other.matrix



from __future__ import absolute_import, division, print_function

from .slice_cluster import SliceCluster
from .slice_filename import SliceFilename

class SliceFilenameLauncher(object):
    @staticmethod
    def run(args):
        SliceFilename(
            args.matrix, args.file_names
        ).run()

class SliceClusterLauncher(object):
    @staticmethod
    def run(args):
        SliceCluster(
            args.matrix, args.annotate_tsv, args.clustersnames
        ).run()



import matplotlib
matplotlib.use('Agg')
import warnings
warnings.filterwarnings("ignore")
import sklearn.manifold as sm
from time import localtime, strftime

from .composers.first_page_composer import FirstPageComposer
from .composers.pie_page_composer import PiePageComposer
from .composers.mds_page_composer import MdsPageComposer

class Rendering:
    def multidimentionnal_scaling(self, matrix, seed):
        # return numpy.random.random((len(matrix), 2))
        return sm.MDS(dissimilarity='precomputed', n_jobs=-1, random_state=seed).fit_transform(matrix.get_matrix())

    def __init__(self, tool_name):
        self.time = strftime("%a, %d %b %Y %H:%M:%S %Z", localtime())
        self.tool_name = tool_name

    def render(self, annotation, title, seed, scatter_plot, ordered_group_names, rescale_heatmap=False):
        matrix = annotation.get_matrix()

        composers = [FirstPageComposer(annotation, title, ordered_group_names, rescale_heatmap)]

        for group_name in ordered_group_names:
            composers.append(PiePageComposer(annotation, title, group_name))

        if scatter_plot:
            p_matrix = self.multidimentionnal_scaling(matrix.to_distance(), seed)
            if len(p_matrix) != len(matrix.get_file_names()):
                raise Exception("Ops! Something is went wrong with the MDS.")

            for group_name in ordered_group_names:
                composers.append(MdsPageComposer(p_matrix, annotation, group_name, title))

        return [composers[i].run(self.make_left_lower_page(),
                                 self.make_rigth_lower_page(i + 1))
                for i in range(len(composers))]

    def make_left_lower_page(self):
        return self.time + '; ' + self.tool_name

    def make_rigth_lower_page(self, page_number):
        return 'Page ' + str(page_number)



from __future__ import absolute_import, division, print_function

from analysis.shared.util.string_builder import StringBuilder
from .annotateapp_builder import AnnotateAppBuilder


class AnnotateLauncher(object):
    @staticmethod
    def run(args):
        ab = AnnotateAppBuilder()
        ab.set_reordered_matrix_filename(args.rmat)
        ab.set_matrix_filename(args.matrix)
        ab.set_metadata_filename(args.metadata)
        ab.set_desired_nb_clusters(args.k)
        ab.set_groups(args.categories)
        ab.set_pdf_file(args.pdf)
        ab.set_title(StringBuilder(args.title))
        ab.set_tsv_file(args.tsv)
        ab.set_seed(args.seed)
        ab.set_mds(args.mds)
        ab.set_linkage_method(args.linkage)
        ab.set_rescale(args.rescale)
        ab.set_md5sum(args.md5sum)

        ab.build().run()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os.path
from setuptools import setup, find_packages
from analysis.config import VERSION, DESCRIPTION, NAME

with open('README.md', 'r') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name = NAME,
    version = VERSION,
    author = "Simon Hébert-Deschamps",
    author_email = "simon.hebert-deschamps@usherbrooke.ca",
    description = DESCRIPTION,
    packages = find_packages(),
    entry_points = {
        'console_scripts': [
            'epigeec-analysis = analysis.main:cli',
        ]
    },
    test_suite='tests',
    install_requires = [
        'matplotlib',
        'numpy',
        'pandas',
        'scipy',
        'sklearn',
        'reportlab',
    ],
    python_requires='>=2.7.5, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, <4',
    long_description = LONG_DESCRIPTION,
    long_description_content_type = 'text/markdown',
    url = 'https://bitbucket.org/labjacquespe/epigeec_analysis',
    classifiers = [
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Intended Audience :: Science/Research',
        'Environment :: Console',
    ],
)